// Updates: 2006.01.13


import java.io.*;
import java.util.*;
import javax.microedition.io.*;
import javax.microedition.io.file.*;
import javax.microedition.lcdui.*;


/**
 * This class provides a user interface to browse a file system in order to 
 * create files or directories, delete them, or select a single file. Requires
 * JSR 75 (PDA Profile for J2ME).
 * 
 * @author Michel Deriaz
 */
public class FileChooser implements CommandListener, Runnable {
  
  /** No choice has been made so far. */
  public static final int NOT_CHOOSEN = -1;
  
  /** The current operation has been canceled. */
  public static final int CANCELED = 0;
  
  /** The user choosed <code>NULL</code>, which means "no file". */ 
  public static final int CHOOSEN_NULL = 1;
  
  /** The user choosed an existing file. */
  public static final int CHOOSEN = 2;

  
  final static int OPERATION_INIT = 0;
  final static int OPERATION_GOTO = 1;
  final static int OPERATION_CHOOSE = 2;
  final static int OPERATION_CREATE_FILE = 3;
  final static int OPERATION_CREATE_DIR = 4; 
  final static int OPERATION_DELETE = 5;   
    
  String filename;
  int state = NOT_CHOOSEN;
  int operation;
  Display display;
  FileConnection fc;
  List list;
  TextBox tb;
  Command cmdBack, cmdCreateFile, cmdOKCreateFile, cmdCreateDir, cmdOKCreateDir,  cmdDelete, cmdOKDelete, cmdCancelDelete;
  

  /**
   * Creates a FileChooser, and sets the current pointer to the specified
   * filename. If <code>initFilename</code> is <code>null</code>, then the 
   * current pointer will be set on the top of all available file systems.
   * @param display the current display
   * @param initFilename the initial filename or <code>null</code> if there is
   * no
   */
  public FileChooser(Display display, String initFilename) {
    this.display = display;
    filename = initFilename;
    list = new List(".", List.IMPLICIT);
    cmdBack = new Command("Back", Command.ITEM, 0);
    cmdCreateFile = new Command("Create file", Command.ITEM, 0);
    cmdOKCreateFile = new Command("OK", Command.OK, 0);
    cmdCreateDir = new Command("Create dir", Command.ITEM, 0);
    cmdOKCreateDir = new Command("OK", Command.OK, 0);
    cmdDelete = new Command("Delete", Command.ITEM, 0);
    cmdOKDelete = new Command("OK", Command.ITEM, 0);
    cmdCancelDelete = new Command("Cancel", Command.ITEM, 0);
    list.addCommand(cmdBack);
    list.setCommandListener(this);
    if (initFilename == null) start(OPERATION_INIT);
    else start(OPERATION_GOTO);
  }
  
  
  /**
   * Gets the choosen filename.
   * @return the choosen filename
   */
  public String getFilename() {
    return filename;
  }
   
  
  /**
   * Gets the current state of the process. The values can be
   * <code>NOT_CHOOSEN</code>, <code>CANCELED</code>,
   * <code>CHOOSEN_NULL</code>, or <code>CHOOSEN</code>.
   * @return the current state of the process
   */
  public int getState() {
    return state;
  }
  
  
  /**
   * Blocks the current Thread until the user chooses a file or cancels the
   * process.
   */
  public void waitUntilFinished() {
    while (state == NOT_CHOOSEN) Thread.yield();
  }
    
  
  public void commandAction(Command c, Displayable s) {
    if (c.equals(cmdBack)) {
      try {
        fc.close();
      }
      catch (Exception e) {}
      state = CANCELED;
    }
    if (c.equals(cmdCreateFile)) {
      tb = new TextBox("Create file", null, 40, TextField.ANY);
      tb.addCommand(cmdOKCreateFile);
      tb.setCommandListener(this);
      display.setCurrent(tb);
    }
    if (c.equals(cmdOKCreateFile)) start(OPERATION_CREATE_FILE);
    if (c.equals(cmdCreateDir)) {
      tb = new TextBox("Create directory", null, 40, TextField.ANY);
      tb.addCommand(cmdOKCreateDir);
      tb.setCommandListener(this);
      display.setCurrent(tb);
    }
    if (c.equals(cmdOKCreateDir)) start(OPERATION_CREATE_DIR);
    if (c.equals(cmdDelete)) {
      Alert alert = new Alert("Delete", "Delete selected file?", null, AlertType.CONFIRMATION);
      alert.addCommand(cmdOKDelete);
      alert.addCommand(cmdCancelDelete);
      alert.setCommandListener(this);
      display.setCurrent(alert, list);
    }
    if (c.equals(cmdOKDelete)) start(OPERATION_DELETE);
    if (c.equals(cmdCancelDelete)) display.setCurrent(list);
    if (c.equals(List.SELECT_COMMAND)) start(OPERATION_CHOOSE);
  }

       
  public void run() {
    display.setCurrent(new Form("Please wait..."));
    if (operation == OPERATION_INIT) {
      list.deleteAll();
      list.setTitle(".");
      Enumeration roots = FileSystemRegistry.listRoots();
      while (roots.hasMoreElements()) {
        list.append("r " + (String)roots.nextElement(), null);
      }
      list.append("- NULL (no file)", null);
      display.setCurrent(list);
    }
    if (operation == OPERATION_GOTO) {
      try {
        fc = (FileConnection)Connector.open("file://" + filename, Connector.READ);
        String name = fc.getName();
        if (!fc.isDirectory()) fc = (FileConnection)Connector.open("file://" + fc.getPath(), Connector.READ);
        updateList();
        for (int i = 0; i < list.size(); i++) {
          if (list.getString(i).equals("f " + name)) list.setSelectedIndex(i, true);
        }
        display.setCurrent(list);
      }
      catch (Exception e) {
        start(OPERATION_INIT);
      }
    }
    if (operation == OPERATION_CHOOSE) {
      try {
        String sel = list.getString(list.getSelectedIndex());
        if (sel.startsWith("f")) {
          filename = fc.getPath() + fc.getName() + sel.substring(2);
          state = CHOOSEN;
          return;
        }  
        else if (sel.startsWith("d")) fc.setFileConnection(sel.substring(2));
        else if (sel.startsWith("r")) fc = (FileConnection)Connector.open("file:///" + sel.substring(2), Connector.READ);
        else if (sel.startsWith("-")) {
          state = CHOOSEN_NULL;
          return;
        }
        updateList();
      }
      // thrown by setFileConnection() when a root is targeted
      catch (IllegalArgumentException e) { 
        start(OPERATION_INIT);
      }
      catch (IOException e) {}
      display.setCurrent(list);
    }
    if (operation == OPERATION_CREATE_FILE) {
      String currentURL = fc.getURL();
      try {
        fc = (FileConnection)Connector.open(currentURL + tb.getString(), Connector.READ_WRITE);
        fc.create();
        list.append("f " + tb.getString(), null);
        list.setSelectedIndex(list.size()-1, true);
        display.setCurrent(list);
      }
      catch (Exception e) {
        Alert alert = new Alert("Error", "Impossible to create the file. Possible reasons include that the file already exists.", null, AlertType.ERROR);
        alert.setTimeout(Alert.FOREVER);
        display.setCurrent(alert, list);
      }
      try {
        fc = (FileConnection)Connector.open(currentURL, Connector.READ);    
      }
      catch (IOException e) {}        
    }
    if (operation == OPERATION_CREATE_DIR) {
      String currentURL = fc.getURL();
      try {
        String file = tb.getString();
        if (!file.endsWith("/")) file += "/";
        fc = (FileConnection)Connector.open(currentURL + file, Connector.READ_WRITE);
        fc.mkdir();
        list.append("d " + file, null);
        list.setSelectedIndex(list.size()-1, true);
        display.setCurrent(list);
      }
      catch (Exception e) {
        Alert alert = new Alert("Error", "Impossible to create the directory. Possible reasons include that the directory already exists.", null, AlertType.ERROR);
        alert.setTimeout(Alert.FOREVER);
        display.setCurrent(alert, list);
      }
      try {
        fc = (FileConnection)Connector.open(currentURL, Connector.READ);    
      }
      catch (IOException e) {}        
    }
    if (operation == OPERATION_DELETE) {
      // if the user tries to delete a node as a first operation (fc still null)
      if (fc == null) {
        display.setCurrent(list);
        return;
      }
      String currentURL = fc.getURL();
      try {
        String sel = list.getString(list.getSelectedIndex());
        if (sel.startsWith("r") || sel.startsWith("-") || sel.startsWith("d ..")) throw new IOException();
        fc = (FileConnection)Connector.open(currentURL + sel.substring(2), Connector.READ_WRITE);
        fc.delete();
        display.setCurrent(list);
      }
      catch (Exception e) {
        Alert alert = new Alert("Error", "Impossible to delete. Possible reasons include either it is a root or a non-empty directory.", null, AlertType.ERROR);
        alert.setTimeout(Alert.FOREVER);
        display.setCurrent(alert, list);
      }
      try {
        fc = (FileConnection)Connector.open(currentURL, Connector.READ);
      }
      catch (IOException e) {}
      updateList();
    }
    
    if (operation == OPERATION_INIT) {
      list.removeCommand(cmdCreateFile);
      list.removeCommand(cmdCreateDir);
      list.removeCommand(cmdDelete);
    }
    else {
      list.addCommand(cmdCreateFile);
      list.addCommand(cmdCreateDir);
      list.addCommand(cmdDelete);
    }
  }
  

  private void start(int operation) {
    this.operation = operation;
    Thread runner = new Thread(this);
    runner.start();
  }
 
  
  private void updateList() {
    list.deleteAll();
    list.append("d ..", null);
    Enumeration files = null;
    try {
      files = fc.list("*", true);
    }
    catch (IOException e) {}
    Vector vDir = new Vector();
    Vector vFiles = new Vector();
    while (files.hasMoreElements()) {
      String file = (String)files.nextElement();
      if (file.endsWith("/")) vDir.addElement("d " + file);
      else vFiles.addElement("f " + file);
    }
    for (int i = 0; i < vDir.size(); i++) list.append((String)vDir.elementAt(i), null);
    for (int i = 0; i < vFiles.size(); i++) list.append((String)vFiles.elementAt(i), null);
    list.setTitle(fc.getPath() + fc.getName());
  }  
}